/*
   Copyright (C) 2009 Roman Belov

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _H_PHYS_
#define _H_PHYS_

#include "draw.h"

typedef struct link link_t;
typedef struct concave concave_t;

enum {
	CONCAVE_CONST		= 0x0010,
	CONCAVE_SOFT		= 0x0020,
	CONCAVE_BEND		= 0x0040,
	CONCAVE_PHANTOM		= 0x0080,
	CONCAVE_TYPE_MASK	= 0x00f0,
	CONCAVE_PHANTOM_RED	= 0x0100,
	CONCAVE_PHANTOM_GREEN	= 0x0200,
	CONCAVE_PHANTOM_BLUE	= 0x0400,
	CONCAVE_PLAYER		= 0x1000
};

struct link {
	point_t		*beg;		/* Begin point */
	point_t		*end;		/* End point */
	float		dist;		/* Target distance */
	float		wght;		/* Weight of <beg> point */
};

struct concave {
	point_t		*last;		/* Last points */
	point_t		*pts;		/* Points */
	int		pt;		/* Points number */
	link_t		*lns;		/* Links */
	int		ln;		/* Links number */
	int		flags;		/* Various flags */
	point_t		*bnd_c;		/* Center of bounding volume */
	float		bnd_r;		/* Bounding radius */
	concave_t	*next;
	concave_t	*prev;
};

concave_t *phys_alloc(int len);
void phys_free(concave_t *cx);
void phys_free_all();

void phys_insert(concave_t *cx);
void phys_remove(concave_t *cx);
void phys_remove_hold();
void phys_remove_back();

concave_t *phys_get_all();

void phys_pick(float px, float py);
void phys_pick_move(float px, float py);
void phys_pick_detach();

void phys_open();
void phys_close();
void phys_update(float dt);
void phys_draw();

#endif /* _H_PHYS_ */

